package kr.co.vividnext.sodalive.search

import android.annotation.SuppressLint
import android.app.Service
import android.content.Intent
import android.graphics.Rect
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.view.View
import android.view.inputmethod.InputMethodManager
import android.widget.Toast
import androidx.annotation.OptIn
import androidx.media3.common.util.UnstableApi
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.google.android.material.tabs.TabLayout
import com.jakewharton.rxbinding4.widget.textChanges
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.audio_content.detail.AudioContentDetailActivity
import kr.co.vividnext.sodalive.audio_content.series.detail.SeriesDetailActivity
import kr.co.vividnext.sodalive.base.BaseActivity
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.common.LoadingDialog
import kr.co.vividnext.sodalive.databinding.ActivitySearchBinding
import kr.co.vividnext.sodalive.explorer.profile.UserProfileActivity
import kr.co.vividnext.sodalive.extensions.dpToPx
import org.koin.android.ext.android.inject
import java.util.concurrent.TimeUnit

class SearchActivity : BaseActivity<ActivitySearchBinding>(ActivitySearchBinding::inflate) {

    private val viewModel: SearchViewModel by inject()

    private lateinit var imm: InputMethodManager
    private lateinit var loadingDialog: LoadingDialog

    private val handler = Handler(Looper.getMainLooper())

    private lateinit var unifiedChannelAdapter: SearchAdapter
    private lateinit var unifiedContentAdapter: SearchAdapter
    private lateinit var unifiedSeriesAdapter: SearchAdapter

    private lateinit var channelAdapter: SearchAdapter
    private lateinit var contentAdapter: SearchAdapter
    private lateinit var seriesAdapter: SearchAdapter

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        imm = getSystemService(
            Service.INPUT_METHOD_SERVICE
        ) as InputMethodManager

        binding.etSearch.requestFocus()
        handler.postDelayed({
            val imm = getSystemService(INPUT_METHOD_SERVICE) as InputMethodManager
            imm.showSoftInput(binding.etSearch, InputMethodManager.SHOW_IMPLICIT)
        }, 500)

        bindData()
    }

    override fun onPause() {
        hideKeyboard()
        super.onPause()
    }

    override fun setupView() {
        loadingDialog = LoadingDialog(this, layoutInflater)
        binding.ivBack.setOnClickListener { finish() }

        setupTabs()
        setupUnifiedView()
        setupChannelListView()
        setupContentListView()
        setupSeriesListView()
    }

    private fun setupChannelListView() {
        channelAdapter = SearchAdapter { clickItem(it) }

        binding.rvCreator.layoutManager = LinearLayoutManager(
            this,
            LinearLayoutManager.VERTICAL,
            false
        )

        binding.rvCreator.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                outRect.left = 13.3f.dpToPx().toInt()
                outRect.right = 13.3f.dpToPx().toInt()
                outRect.top = 13.3f.dpToPx().toInt()
                outRect.bottom = 13.3f.dpToPx().toInt()
            }
        })

        binding.rvCreator.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                super.onScrolled(recyclerView, dx, dy)

                val layoutManager = recyclerView.layoutManager as? LinearLayoutManager
                if (
                    layoutManager != null &&
                    layoutManager
                        .findLastCompletelyVisibleItemPosition() == channelAdapter.itemCount - 1
                ) {
                    viewModel.searchCreatorList()
                }
            }
        })

        binding.rvCreator.adapter = channelAdapter
    }

    private fun setupContentListView() {
        contentAdapter = SearchAdapter { clickItem(it) }

        binding.rvContent.layoutManager = LinearLayoutManager(
            this,
            LinearLayoutManager.VERTICAL,
            false
        )

        binding.rvContent.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                outRect.left = 13.3f.dpToPx().toInt()
                outRect.right = 13.3f.dpToPx().toInt()
                outRect.top = 13.3f.dpToPx().toInt()
                outRect.bottom = 13.3f.dpToPx().toInt()
            }
        })

        binding.rvContent.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                super.onScrolled(recyclerView, dx, dy)

                val layoutManager = recyclerView.layoutManager as? LinearLayoutManager
                if (
                    layoutManager != null &&
                    layoutManager
                        .findLastCompletelyVisibleItemPosition() == contentAdapter.itemCount - 1
                ) {
                    viewModel.searchContentList()
                }
            }
        })

        binding.rvContent.adapter = contentAdapter
    }

    private fun setupSeriesListView() {
        seriesAdapter = SearchAdapter { clickItem(it) }

        binding.rvSeries.layoutManager = LinearLayoutManager(
            this,
            LinearLayoutManager.VERTICAL,
            false
        )

        binding.rvSeries.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                outRect.left = 13.3f.dpToPx().toInt()
                outRect.right = 13.3f.dpToPx().toInt()
                outRect.top = 13.3f.dpToPx().toInt()
                outRect.bottom = 13.3f.dpToPx().toInt()
            }
        })

        binding.rvSeries.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                super.onScrolled(recyclerView, dx, dy)

                val layoutManager = recyclerView.layoutManager as? LinearLayoutManager
                if (
                    layoutManager != null &&
                    layoutManager
                        .findLastCompletelyVisibleItemPosition() == seriesAdapter.itemCount - 1
                ) {
                    viewModel.searchSeriesList()
                }
            }
        })

        binding.rvSeries.adapter = seriesAdapter
    }

    private fun setupUnifiedView() {
        unifiedChannelAdapter = SearchAdapter { clickItem(it) }
        unifiedContentAdapter = SearchAdapter { clickItem(it) }
        unifiedSeriesAdapter = SearchAdapter { clickItem(it) }

        binding.tvMoreCreator.setOnClickListener {
            viewModel.changeTab(SearchViewModel.SearchPageTab.CREATOR)
        }

        binding.tvMoreContent.setOnClickListener {
            viewModel.changeTab(SearchViewModel.SearchPageTab.CONTENT)
        }

        binding.tvMoreSeries.setOnClickListener {
            viewModel.changeTab(SearchViewModel.SearchPageTab.SERIES)
        }

        binding.rvUnifiedCreator.layoutManager = LinearLayoutManager(
            this,
            LinearLayoutManager.VERTICAL,
            false
        )
        binding.rvUnifiedCreator.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                outRect.left = 13.3f.dpToPx().toInt()
                outRect.right = 13.3f.dpToPx().toInt()
                outRect.top = 13.3f.dpToPx().toInt()
                outRect.bottom = 13.3f.dpToPx().toInt()
            }
        })
        binding.rvUnifiedCreator.adapter = unifiedChannelAdapter

        binding.rvUnifiedContent.layoutManager = LinearLayoutManager(
            this,
            LinearLayoutManager.VERTICAL,
            false
        )
        binding.rvUnifiedContent.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                outRect.left = 13.3f.dpToPx().toInt()
                outRect.right = 13.3f.dpToPx().toInt()
                outRect.top = 13.3f.dpToPx().toInt()
                outRect.bottom = 13.3f.dpToPx().toInt()
            }
        })
        binding.rvUnifiedContent.adapter = unifiedContentAdapter

        binding.rvUnifiedSeries.layoutManager = LinearLayoutManager(
            this,
            LinearLayoutManager.VERTICAL,
            false
        )
        binding.rvUnifiedSeries.addItemDecoration(object : RecyclerView.ItemDecoration() {
            override fun getItemOffsets(
                outRect: Rect,
                view: View,
                parent: RecyclerView,
                state: RecyclerView.State
            ) {
                super.getItemOffsets(outRect, view, parent, state)

                outRect.left = 13.3f.dpToPx().toInt()
                outRect.right = 13.3f.dpToPx().toInt()
                outRect.top = 13.3f.dpToPx().toInt()
                outRect.bottom = 13.3f.dpToPx().toInt()
            }
        })
        binding.rvUnifiedSeries.adapter = unifiedSeriesAdapter
    }

    @OptIn(UnstableApi::class)
    private fun clickItem(item: SearchResponseItem) {
        hideKeyboard()

        startActivity(
            when (item.type) {
                SearchResponseType.CREATOR -> {
                    Intent(applicationContext, UserProfileActivity::class.java).apply {
                        putExtra(Constants.EXTRA_USER_ID, item.id)
                    }
                }

                SearchResponseType.CONTENT -> {
                    Intent(applicationContext, AudioContentDetailActivity::class.java).apply {
                        putExtra(Constants.EXTRA_AUDIO_CONTENT_ID, item.id)
                    }
                }

                SearchResponseType.SERIES -> {
                    Intent(applicationContext, SeriesDetailActivity::class.java).apply {
                        putExtra(Constants.EXTRA_SERIES_ID, item.id)
                    }
                }
            }
        )
    }

    private fun setupTabs() {
        val tabs = binding.tabs
        val tabTitles = listOf("통합", "채널", "콘텐츠", "시리즈")
        for (title in tabTitles) {
            tabs.addTab(tabs.newTab().setText(title))
        }

        tabs.addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
            override fun onTabSelected(tab: TabLayout.Tab) {
                val selectedTab = SearchViewModel.SearchPageTab.fromOrdinal(tab.position)
                viewModel.changeTab(selectedTab!!)

                tab.view.isSelected = true
                hideKeyboard()
            }

            override fun onTabUnselected(tab: TabLayout.Tab) {
                tab.view.isSelected = false
            }

            override fun onTabReselected(tab: TabLayout.Tab) {
            }
        })
    }

    @SuppressLint("NotifyDataSetChanged")
    private fun bindData() {
        compositeDisposable.add(
            binding.etSearch.textChanges().skip(1)
                .debounce(500, TimeUnit.MILLISECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribeOn(Schedulers.io())
                .subscribe {
                    channelAdapter.clear()
                    contentAdapter.clear()
                    seriesAdapter.clear()

                    unifiedChannelAdapter.clear()
                    unifiedContentAdapter.clear()
                    unifiedSeriesAdapter.clear()

                    viewModel.keyword = it.toString()
                    if (it.length >= 2) {
                        viewModel.searchUnified()
                        binding.tabs.visibility = View.VISIBLE
                        binding.tvResultX.visibility = View.GONE
                    } else {
                        binding.nsSearchUnified.visibility = View.GONE
                        binding.rvCreator.visibility = View.GONE
                        binding.rvContent.visibility = View.GONE
                        binding.rvSeries.visibility = View.GONE
                        binding.tabs.visibility = View.GONE
                        binding.tvResultX.visibility = View.GONE
                    }
                }
        )

        viewModel.toastLiveData.observe(this) {
            it?.let { Toast.makeText(this@SearchActivity, it, Toast.LENGTH_LONG).show() }
        }

        viewModel.isLoading.observe(this) {
            if (it) {
                loadingDialog.show(screenWidth, "")
            } else {
                loadingDialog.dismiss()
            }
        }

        viewModel.searchUnifiedLiveData.observe(this) {
            if (
                it.creatorList.isEmpty() &&
                it.seriesList.isEmpty() &&
                it.contentList.isEmpty()
            ) {
                binding.tabs.visibility = View.GONE
                hideAllView()
                if (viewModel.keyword.isNotBlank()) {
                    binding.tvResultX.visibility = View.VISIBLE
                } else {
                    binding.tvResultX.visibility = View.GONE
                }
            } else {
                binding.tabs.visibility = View.VISIBLE
                binding.nsSearchUnified.visibility = View.VISIBLE

                if (it.creatorList.isNotEmpty()) {
                    binding.tvCreatorTitle.visibility = View.VISIBLE
                    binding.tvMoreCreator.visibility = View.VISIBLE
                    binding.rvUnifiedCreator.visibility = View.VISIBLE
                    unifiedChannelAdapter.items.addAll(it.creatorList)
                    unifiedChannelAdapter.notifyDataSetChanged()
                } else {
                    binding.tvCreatorTitle.visibility = View.GONE
                    binding.tvMoreCreator.visibility = View.GONE
                    binding.rvUnifiedCreator.visibility = View.GONE
                }

                if (it.contentList.isNotEmpty()) {
                    binding.tvContentTitle.visibility = View.VISIBLE
                    binding.tvMoreContent.visibility = View.VISIBLE
                    binding.rvUnifiedContent.visibility = View.VISIBLE
                    unifiedContentAdapter.items.addAll(it.contentList)
                    unifiedContentAdapter.notifyDataSetChanged()
                } else {
                    binding.tvContentTitle.visibility = View.GONE
                    binding.tvMoreContent.visibility = View.GONE
                    binding.rvUnifiedContent.visibility = View.GONE
                }

                if (it.seriesList.isNotEmpty()) {
                    binding.tvSeriesTitle.visibility = View.VISIBLE
                    binding.tvMoreSeries.visibility = View.VISIBLE
                    binding.rvUnifiedSeries.visibility = View.VISIBLE
                    unifiedSeriesAdapter.items.addAll(it.seriesList)
                    unifiedSeriesAdapter.notifyDataSetChanged()
                } else {
                    binding.tvSeriesTitle.visibility = View.GONE
                    binding.tvMoreSeries.visibility = View.GONE
                    binding.rvUnifiedSeries.visibility = View.GONE
                }
            }
        }

        viewModel.searchCreatorLiveData.observe(this) {
            channelAdapter.items.addAll(it)
            channelAdapter.notifyDataSetChanged()

            hideAllView()
            if (channelAdapter.items.isEmpty()) {
                if (viewModel.keyword.isNotBlank()) {
                    binding.tvResultX.visibility = View.VISIBLE
                } else {
                    binding.tvResultX.visibility = View.GONE
                }
            } else {
                binding.rvCreator.visibility = View.VISIBLE
            }
        }

        viewModel.searchContentLiveData.observe(this) {
            contentAdapter.items.addAll(it)
            contentAdapter.notifyDataSetChanged()

            hideAllView()
            if (contentAdapter.items.isEmpty()) {
                if (viewModel.keyword.isNotBlank()) {
                    binding.tvResultX.visibility = View.VISIBLE
                } else {
                    binding.tvResultX.visibility = View.GONE
                }
            } else {
                binding.rvContent.visibility = View.VISIBLE
            }
        }

        viewModel.searchSeriesLiveData.observe(this) {
            seriesAdapter.items.addAll(it)
            seriesAdapter.notifyDataSetChanged()

            hideAllView()
            if (seriesAdapter.items.isEmpty()) {
                if (viewModel.keyword.isNotBlank()) {
                    binding.tvResultX.visibility = View.VISIBLE
                } else {
                    binding.tvResultX.visibility = View.GONE
                }
            } else {
                binding.rvSeries.visibility = View.VISIBLE
            }
        }

        viewModel.currentTabLiveData.observe(this) { currentTab ->
            hideAllView()

            binding.tabs.getTabAt(currentTab.ordinal)?.select()
            when (currentTab) {
                SearchViewModel.SearchPageTab.CREATOR -> {
                    if (channelAdapter.items.isEmpty()) {
                        viewModel.searchCreatorList()
                    } else {
                        binding.rvCreator.visibility = View.VISIBLE
                    }
                }

                SearchViewModel.SearchPageTab.CONTENT -> {
                    if (contentAdapter.items.isEmpty()) {
                        viewModel.searchContentList()
                    } else {
                        binding.rvContent.visibility = View.VISIBLE
                    }
                }

                SearchViewModel.SearchPageTab.SERIES -> {
                    if (seriesAdapter.items.isEmpty()) {
                        viewModel.searchSeriesList()
                    } else {
                        binding.rvSeries.visibility = View.VISIBLE
                    }
                }

                else -> {
                    if (
                        unifiedChannelAdapter.items.isEmpty() &&
                        unifiedContentAdapter.items.isEmpty() &&
                        unifiedSeriesAdapter.items.isEmpty()
                    ) {
                        if (viewModel.keyword.isNotBlank()) {
                            binding.tvResultX.visibility = View.VISIBLE
                        } else {
                            binding.tvResultX.visibility = View.GONE
                        }
                    } else {
                        binding.nsSearchUnified.visibility = View.VISIBLE
                    }
                }
            }
        }
    }

    private fun hideAllView() {
        binding.nsSearchUnified.visibility = View.GONE
        binding.rvCreator.visibility = View.GONE
        binding.rvContent.visibility = View.GONE
        binding.rvSeries.visibility = View.GONE
        binding.tvResultX.visibility = View.GONE
    }

    private fun hideKeyboard() {
        handler.postDelayed({
            imm.hideSoftInputFromWindow(
                window.decorView.applicationWindowToken,
                InputMethodManager.HIDE_NOT_ALWAYS
            )
        }, 100)
    }
}
