package kr.co.vividnext.sodalive.search

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kr.co.vividnext.sodalive.base.BaseViewModel
import kr.co.vividnext.sodalive.common.SharedPreferenceManager

class SearchViewModel(
    private val repository: SearchRepository
) : BaseViewModel() {
    enum class SearchPageTab {
        UNIFIED, CREATOR, CONTENT, SERIES;

        companion object {
            fun fromOrdinal(ordinal: Int): SearchPageTab? {
                return SearchPageTab.values().getOrNull(ordinal)
            }
        }
    }

    var keyword = ""

    private val _currentTabLiveData = MutableLiveData(SearchPageTab.UNIFIED)
    val currentTabLiveData: LiveData<SearchPageTab>
        get() = _currentTabLiveData

    private val _toastLiveData = MutableLiveData<String?>()
    val toastLiveData: LiveData<String?>
        get() = _toastLiveData

    private var _isLoading = MutableLiveData(false)
    val isLoading: LiveData<Boolean>
        get() = _isLoading

    private var _searchUnifiedLiveData = MutableLiveData<SearchUnifiedResponse>()
    val searchUnifiedLiveData: LiveData<SearchUnifiedResponse>
        get() = _searchUnifiedLiveData

    private var _searchCreatorLiveData = MutableLiveData<List<SearchResponseItem>>()
    val searchCreatorLiveData: LiveData<List<SearchResponseItem>>
        get() = _searchCreatorLiveData

    private var _searchContentLiveData = MutableLiveData<List<SearchResponseItem>>()
    val searchContentLiveData: LiveData<List<SearchResponseItem>>
        get() = _searchContentLiveData

    private var _searchSeriesLiveData = MutableLiveData<List<SearchResponseItem>>()
    val searchSeriesLiveData: LiveData<List<SearchResponseItem>>
        get() = _searchSeriesLiveData

    private var searchCreatorPage = 1
    private var searchContentPage = 1
    private var searchSeriesPage = 1

    private var isSearchCreatorLast = false
    private var isSearchContentLast = false
    private var isSearchSeriesLast = false

    private val size = 20

    fun changeTab(tab: SearchPageTab) {
        _currentTabLiveData.value = tab
    }

    fun searchUnified() {
        if (!_isLoading.value!!) {
            _currentTabLiveData.value = SearchPageTab.UNIFIED

            searchCreatorPage = 1
            searchContentPage = 1
            searchSeriesPage = 1

            isSearchCreatorLast = false
            isSearchContentLast = false
            isSearchSeriesLast = false

            _isLoading.value = true
            compositeDisposable.add(
                repository.searchUnified(
                    keyword = keyword,
                    token = "Bearer ${SharedPreferenceManager.token}"
                )
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(
                        {
                            _isLoading.value = false
                            if (it.success && it.data != null) {
                                _searchUnifiedLiveData.value = it.data!!
                            } else {
                                if (it.message != null) {
                                    _toastLiveData.value = it.message
                                } else {
                                    _toastLiveData
                                        .value = "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                }
                            }
                        },
                        {
                            _isLoading.value = false
                            it.message?.let { message -> Logger.e(message) }
                            _toastLiveData
                                .value = "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                        }
                    )
            )
        }
    }

    fun searchCreatorList() {
        if (!_isLoading.value!! && !isSearchCreatorLast) {
            _isLoading.value = true
            compositeDisposable.add(
                repository.searchCreatorList(
                    keyword = keyword,
                    page = searchCreatorPage,
                    size = size,
                    token = "Bearer ${SharedPreferenceManager.token}"
                )
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(
                        {
                            _isLoading.value = false
                            if (it.success && it.data != null) {
                                searchCreatorPage += 1

                                val data = it.data
                                _searchCreatorLiveData.value = data.items

                                if (data.items.isEmpty()) {
                                    isSearchCreatorLast = true
                                }
                            } else {
                                if (it.message != null) {
                                    _toastLiveData.value = it.message
                                } else {
                                    _toastLiveData
                                        .value = "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                }
                            }
                        },
                        {
                            _isLoading.value = false
                            it.message?.let { message -> Logger.e(message) }
                            _toastLiveData
                                .value = "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                        }
                    )
            )
        } else {
            _searchCreatorLiveData.value = emptyList()
        }
    }

    fun searchContentList() {
        if (!_isLoading.value!! && !isSearchContentLast) {
            _isLoading.value = true
            compositeDisposable.add(
                repository.searchContentList(
                    keyword = keyword,
                    page = searchContentPage,
                    size = size,
                    token = "Bearer ${SharedPreferenceManager.token}"
                )
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(
                        {
                            _isLoading.value = false
                            if (it.success && it.data != null) {
                                searchContentPage += 1

                                val data = it.data
                                _searchContentLiveData.value = data.items

                                if (data.items.isEmpty()) {
                                    isSearchContentLast = true
                                }
                            } else {
                                if (it.message != null) {
                                    _toastLiveData.value = it.message
                                } else {
                                    _toastLiveData
                                        .value = "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                }
                            }
                        },
                        {
                            _isLoading.value = false
                            it.message?.let { message -> Logger.e(message) }
                            _toastLiveData
                                .value = "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                        }
                    )
            )
        } else {
            _searchContentLiveData.value = emptyList()
        }
    }

    fun searchSeriesList() {
        if (!_isLoading.value!! && !isSearchSeriesLast) {
            _isLoading.value = true
            compositeDisposable.add(
                repository.searchSeriesList(
                    keyword = keyword,
                    page = searchSeriesPage,
                    size = size,
                    token = "Bearer ${SharedPreferenceManager.token}"
                )
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(
                        {
                            _isLoading.value = false
                            if (it.success && it.data != null) {
                                searchSeriesPage += 1

                                val data = it.data
                                _searchSeriesLiveData.value = data.items

                                if (data.items.isEmpty()) {
                                    isSearchSeriesLast = true
                                }
                            } else {
                                if (it.message != null) {
                                    _toastLiveData.value = it.message
                                } else {
                                    _toastLiveData
                                        .value = "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                                }
                            }
                        },
                        {
                            _isLoading.value = false
                            it.message?.let { message -> Logger.e(message) }
                            _toastLiveData
                                .value = "알 수 없는 오류가 발생했습니다. 다시 시도해 주세요."
                        }
                    )
            )
        } else {
            _searchSeriesLiveData.value = emptyList()
        }
    }
}
