package kr.co.vividnext.sodalive.user.login

import android.app.Service
import android.content.Intent
import android.graphics.Rect
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.transition.TransitionManager
import android.view.inputmethod.EditorInfo
import android.view.inputmethod.InputMethodManager
import android.widget.LinearLayout
import android.widget.Toast
import androidx.annotation.OptIn
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.constraintlayout.widget.ConstraintSet
import androidx.credentials.Credential
import androidx.credentials.CredentialManager
import androidx.credentials.CustomCredential
import androidx.credentials.GetCredentialRequest
import androidx.credentials.exceptions.GetCredentialException
import androidx.lifecycle.lifecycleScope
import androidx.media3.common.util.UnstableApi
import com.google.android.libraries.identity.googleid.GetGoogleIdOption
import com.google.android.libraries.identity.googleid.GoogleIdTokenCredential
import com.google.android.libraries.identity.googleid.GoogleIdTokenCredential.Companion.TYPE_GOOGLE_ID_TOKEN_CREDENTIAL
import com.jakewharton.rxbinding4.widget.textChanges
import com.kakao.sdk.auth.model.OAuthToken
import com.kakao.sdk.common.model.ClientError
import com.kakao.sdk.common.model.ClientErrorCause
import com.kakao.sdk.user.UserApiClient
import com.orhanobut.logger.Logger
import io.reactivex.rxjava3.android.schedulers.AndroidSchedulers
import io.reactivex.rxjava3.schedulers.Schedulers
import kotlinx.coroutines.launch
import kr.co.vividnext.sodalive.BuildConfig
import kr.co.vividnext.sodalive.base.BaseActivity
import kr.co.vividnext.sodalive.common.Constants
import kr.co.vividnext.sodalive.common.LoadingDialog
import kr.co.vividnext.sodalive.databinding.ActivityLoginBinding
import kr.co.vividnext.sodalive.extensions.dpToPx
import kr.co.vividnext.sodalive.main.MainActivity
import kr.co.vividnext.sodalive.user.find_password.FindPasswordActivity
import kr.co.vividnext.sodalive.user.signup.SignUpActivity
import org.koin.android.ext.android.inject

@OptIn(UnstableApi::class)
class LoginActivity : BaseActivity<ActivityLoginBinding>(ActivityLoginBinding::inflate) {

    private val viewModel: LoginViewModel by inject()

    private lateinit var imm: InputMethodManager
    private lateinit var loadingDialog: LoadingDialog

    private val handler = Handler(Looper.getMainLooper())

    override fun onCreate(savedInstanceState: Bundle?) {
        imm = getSystemService(Service.INPUT_METHOD_SERVICE) as InputMethodManager

        super.onCreate(savedInstanceState)
        bindData()
    }

    override fun setupView() {
        binding.root.viewTreeObserver.addOnGlobalLayoutListener {
            val rect = Rect()
            binding.root.getWindowVisibleDisplayFrame(rect)

            val keypadHeight = screenHeight - rect.bottom
            if (keypadHeight > screenHeight * 0.15) {
                updateMarginWithAnimation(
                    binding.llLoginContainer,
                    -100f.dpToPx().toInt()
                )
            } else {
                updateMarginWithAnimation(binding.llLoginContainer, 0)
            }
        }

        binding.toolbar.tvBack.text = "로그인"
        binding.toolbar.tvBack.setOnClickListener { finish() }
        loadingDialog = LoadingDialog(this, layoutInflater)

        binding.etPassword.setOnEditorActionListener { _, actionId, _ ->
            if (actionId == EditorInfo.IME_ACTION_DONE) {
                hideKeyboard()
                login()
                true
            } else {
                false
            }
        }
        binding.tvLogin.setOnClickListener {
            hideKeyboard()
            login()
        }

        binding.tvForgotPassword.setOnClickListener {
            startActivity(
                Intent(
                    applicationContext,
                    FindPasswordActivity::class.java
                )
            )
        }

        binding.tvSignUp.setOnClickListener { startSignUp() }
        binding.ivSignUpEmail.setOnClickListener { startSignUp() }

        binding.ivLoginGoogle.setOnClickListener {
            loadingDialog.show(width = screenWidth)
            val credentialManager = CredentialManager.create(this)

            val googleIdOption = GetGoogleIdOption.Builder()
                .setServerClientId(BuildConfig.GOOGLE_CLIENT_ID)
                .setFilterByAuthorizedAccounts(false)
                .setAutoSelectEnabled(false)
                .build()

            // Create the Credential Manager request
            val request = GetCredentialRequest.Builder()
                .addCredentialOption(googleIdOption)
                .build()

            lifecycleScope.launch {
                try {
                    // Launch Credential Manager UI
                    val result = credentialManager.getCredential(
                        context = this@LoginActivity,
                        request = request
                    )
                    loadingDialog.dismiss()

                    // Extract credential from the result returned by Credential Manager
                    handleSignIn(result.credential)
                } catch (e: GetCredentialException) {
                    showToast("구글 로그인을 하지 못했습니다. 다시 시도해 주세요")
                    Logger.e("Couldn't retrieve user's credentials: ${e.localizedMessage}")
                    loadingDialog.dismiss()
                }
            }
        }

        binding.ivLoginKakao.setOnClickListener { loginKakao() }
    }

    private fun handleSignIn(credential: Credential) {
        // Check if credential is of type Google ID
        if (credential is CustomCredential && credential.type == TYPE_GOOGLE_ID_TOKEN_CREDENTIAL) {
            // Create Google ID Token
            val googleIdTokenCredential = GoogleIdTokenCredential.createFrom(credential.data)
            viewModel.googleLogin(idToken = googleIdTokenCredential.idToken) {
                finishAffinity()
                val nextIntent = Intent(applicationContext, MainActivity::class.java)
                val extras = intent.getBundleExtra(Constants.EXTRA_DATA)
                    ?: if (intent.extras != null) {
                        intent.extras
                    } else {
                        null
                    }
                if (extras != null) {
                    nextIntent.putExtra(Constants.EXTRA_DATA, extras)
                }
                intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
                intent.addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP)
                startActivity(nextIntent)
            }
        } else {
            showToast("로그인을 하지 못했습니다. 다시 시도해 주세요")
            Logger.e("Credential is not of type Google ID!")
        }
    }

    private fun login() {
        viewModel.login {
            it?.let { Toast.makeText(applicationContext, it, Toast.LENGTH_LONG).show() }
            finishAffinity()
            val nextIntent = Intent(applicationContext, MainActivity::class.java)
            val extras = intent.getBundleExtra(Constants.EXTRA_DATA)
                ?: if (intent.extras != null) {
                    intent.extras
                } else {
                    null
                }
            if (extras != null) {
                nextIntent.putExtra(Constants.EXTRA_DATA, extras)
            }
            intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
            intent.addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP)
            startActivity(nextIntent)
        }
    }

    private fun bindData() {
        compositeDisposable.add(
            binding.etEmail.textChanges().skip(1)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    viewModel.email = it.toString()
                }
        )

        compositeDisposable.add(
            binding.etPassword.textChanges().skip(1)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    viewModel.password = it.toString()
                }
        )

        viewModel.toastLiveData.observe(this) {
            it?.let { Toast.makeText(applicationContext, it, Toast.LENGTH_LONG).show() }
        }

        viewModel.isLoading.observe(this) {
            if (it) {
                loadingDialog.show(screenWidth)
            } else {
                loadingDialog.dismiss()
            }
        }
    }

    private fun updateMarginWithAnimation(view: LinearLayout, newMargin: Int) {
        val constraintLayout = view.parent as ConstraintLayout
        val constraintSet = ConstraintSet()
        constraintSet.clone(constraintLayout)

        // 변경할 Constraint 적용 (margin 변경)
        constraintSet.connect(
            view.id,
            ConstraintSet.TOP,
            ConstraintSet.PARENT_ID,
            ConstraintSet.TOP,
            newMargin
        )

        // 애니메이션 적용
        TransitionManager.beginDelayedTransition(constraintLayout)
        constraintSet.applyTo(constraintLayout)
    }

    private fun hideKeyboard() {
        handler.postDelayed({
            imm.hideSoftInputFromWindow(
                window.decorView.applicationWindowToken,
                InputMethodManager.HIDE_NOT_ALWAYS
            )
        }, 100)
    }

    private val kakaoLoginCallback: (OAuthToken?, Throwable?) -> Unit = { token, error ->
        loadingDialog.dismiss()
        if (error != null) {
            showToast("카카오 계정으로 로그인 하지 못했습니다. 다시 시도해 주세요")
        } else if (token != null) {
            handleKakaoLogin(token)
        }
    }

    private fun loginKakao() {
        loadingDialog.show(screenWidth)
        if (UserApiClient.instance.isKakaoTalkLoginAvailable(this@LoginActivity)) {
            UserApiClient.instance.loginWithKakaoTalk(this@LoginActivity) { token, error ->
                if (error != null) {
                    Logger.e("error: ${error.message}, ${error.cause}")
                    if (error is ClientError && error.reason == ClientErrorCause.Cancelled) {
                        loadingDialog.dismiss()
                        return@loginWithKakaoTalk
                    } else {
                        UserApiClient.instance.loginWithKakaoAccount(
                            this@LoginActivity,
                            callback = kakaoLoginCallback
                        )
                    }
                } else if (token != null) {
                    handleKakaoLogin(token)
                }
            }
        } else {
            UserApiClient.instance.loginWithKakaoAccount(
                this@LoginActivity,
                callback = kakaoLoginCallback
            )
        }
    }

    private fun handleKakaoLogin(token: OAuthToken) {
        viewModel.kakaoLogin(accessToken = token.accessToken) {
            finishAffinity()
            val nextIntent = Intent(applicationContext, MainActivity::class.java)
            val extras = intent.getBundleExtra(Constants.EXTRA_DATA)
                ?: if (intent.extras != null) {
                    intent.extras
                } else {
                    null
                }
            if (extras != null) {
                nextIntent.putExtra(Constants.EXTRA_DATA, extras)
            }
            intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
            intent.addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP)
            startActivity(nextIntent)
        }
    }

    private fun startSignUp() {
        val nextIntent = Intent(applicationContext, SignUpActivity::class.java)
        val extras = intent.getBundleExtra(Constants.EXTRA_DATA)
            ?: if (intent.extras != null) {
                intent.extras
            } else {
                null
            }
        if (extras != null) {
            nextIntent.putExtra(Constants.EXTRA_DATA, extras)
        }
        startActivity(nextIntent)
    }
}
